"""Conversion between regular and rotated spherical coordinates.
Original Source: Fortran SUBROUTINE regrof.F, by J.E. Haugen, 1992.
"""
from __future__ import division
import numpy as np
import sys

__author__ = "Kasper Hintz"
__credits__ = ["J.E. Haugen"]
__version__ = "0.2"
__email__ = "kah@dmi.dk"

pi = np.pi
rad = pi / 180.
radinv = 1. / rad


def rot_to_reg(pole_lon, pole_lat, lon, lat):
    """Rotates from rotated grid to regular grid

    Parameters
    ----------
    pole_lon : float
        Longitude of pole in degrees
    pole_lat : float
        Latitude of pole in degrees
    lon : float, list
        Longitudes of points in grid. Can also be a single point.
    lat : float, list
        Latitudes of points in grid. Can also be a single point.

    Returns
    -------
    lon1 : float, list
        Longitudes of regular grid. Same shape as input.
    lat1 : float, list
        Latitudes  of regular grid. Same shape as input.
    """
    sin_pole = np.sin(rad * (pole_lat + 90.))
    cos_pole = np.cos(rad * (pole_lat + 90.))

    sin_lon = np.sin(rad * lon)
    cos_lon = np.cos(rad * lon)
    sin_lat = np.sin(rad * lat)
    cos_lat = np.cos(rad * lat)

    lat_tmp = cos_pole * sin_lat + sin_pole * cos_lat * cos_lon

    try:
        lat_tmp[np.where(lat_tmp < -1.)] = -1.
        lat_tmp[np.where(lat_tmp > 1.)] = 1.
    except TypeError:
        if lat_tmp < -1.:
            lat_tmp = -1.
        if lat_tmp > 1.:
            lat_tmp = 1.

    lat2 = np.arcsin(lat_tmp) * radinv

    cos_lat2 = np.cos(lat2 * rad)

    cos_tmp = (cos_pole * cos_lat * cos_lon - sin_pole * sin_lat) / cos_lat2

    try:
        cos_tmp[np.where(cos_tmp < -1.)] = -1.
        cos_tmp[np.where(cos_tmp > 1.)] = 1.
    except TypeError:
        if cos_tmp < -1.:
            cos_tmp = -1.
        if cos_tmp > 1.:
            cos_tmp = 1.

    tmp_sin_lon = cos_lat * sin_lon / cos_lat2
    tmp_cos_lon = np.arccos(cos_tmp) * radinv

    try:
        tmp_cos_lon[np.where(tmp_sin_lon < 0.)] = - \
            tmp_cos_lon[np.where(tmp_sin_lon < 0.)]
    except IndexError:
        if tmp_sin_lon < 0.:
            tmp_cos_lon = -tmp_cos_lon

    lon2 = tmp_cos_lon + pole_lon

    return lon2, lat2


def reg_to_rot(pole_lon, pole_lat, lon, lat):
    """Rotates from regular grid to rotated grid

    Parameters
    ----------
    pole_lon : float
        Longitude of pole in degrees
    pole_lat : float
        Latitude of pole in degrees
    lon : float, list
        Longitudes of points in grid. Can also be a single point.
    lat : float, list
        Latitudes of points in grid. Can also be a single point.

    Returns
    -------
    lon1 : float, list
        Longitudes of rotated grid. Same shape as input.
    lat1 : float, list
        Latitudes  of rotated grid. Same shape as input.

    """
    sin_pole = np.sin(rad * (pole_lat + 90.))
    cos_pole = np.cos(rad * (pole_lat + 90.))

    tmp_lon = rad * (lon - pole_lon)

    sin_lon = np.sin(tmp_lon)
    cos_lon = np.cos(tmp_lon)
    sin_lat = np.sin(rad * lat)
    cos_lat = np.cos(rad * lat)

    lat_tmp = cos_pole * sin_lat - sin_pole * cos_lat * cos_lon

    try:
        lat_tmp[np.where(lat_tmp < -1.)] = -1.
        lat_tmp[np.where(lat_tmp > 1.)] = 1.
    except TypeError:
        if lat_tmp < -1:
            lat_tmp = -1.
        if lat_tmp > 1:
            lat_tmp = 1.

    lat2 = np.arcsin(lat_tmp) * radinv

    cos_tmp = np.cos(lat2 * rad)

    tmp_lon_rot = (cos_pole * cos_lat * cos_lon + sin_pole * sin_lat) / cos_tmp

    try:
        tmp_lon_rot[np.where(tmp_lon_rot < -1.)] = -1.
        tmp_lon_rot[np.where(tmp_lon_rot > 1.)] = 1.
    except TypeError:
        if tmp_lon_rot < -1.:
            tmp_lon_rot = -1.
        if tmp_lon_rot > 1.:
            tmp_lon_rot = 1.

    rot_check = cos_lat * sin_lon / cos_tmp

    lon2 = np.arccos(tmp_lon_rot) * radinv

    try:
        lon2[np.where(rot_check < 0.)] = -lon2[np.where(rot_check < 0.)]
    except IndexError:
        if rot_check < 0.:
            lon2 = -lon2

    return lon2, lat2



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Master module for dmit.dmitio
"""

__author__ = "K. Hintz"
__copyright__ = "Danish Meteorological Institute"

__license__ = "MIT"
__version__ = "0.0.2.2"
__maintainer__ = "K. Hintz"
__email__ = "kah@dmi.dk"
__status__ = "Development"

import sys
import os
sys.path.insert(0, os.path.abspath('./dmitio/'))
import argparse
from argparse import ArgumentDefaultsHelpFormatter

from dmit import ostools
from .arguments import arguments
from .grib import readgrib
from .netcdf import writenc

class MyParser(argparse.ArgumentParser):
    def error(self, message):
        sys.stderr.write('error: %s\n' % message)
        self.print_help()
        sys.exit(2)


def find_files_to_read(args:argparse.Namespace) -> list:
        """Finds the file(s) to read and plot from

        Parameters
        ----------
        args : argparse.Namespace
            Input arguments to dlotter.__main__

        Returns
        -------
        list
            List with path(s) to file(s)
        """

        directory = args.directory

        if args.limit_files > 0:
            inorder = True
        else:
            inorder = False

        files = ostools.find_files(directory, 
                                   prefix=args.prefix, 
                                   postfix=args.postfix,
                                   recursive=False, 
                                   onlyfiles=True,
                                   fullpath=True,
                                   olderthan=None,
                                   inorder=inorder)

        if args.limit_files > 0: 
            if args.limit_files >= len(files):
                limit = len(files)
            else:
                limit = args.limit_files

            files = files[0:limit]

        return files


if __name__ == '__main__':

    modargs = arguments()
    args = modargs.get_args(sys.argv)

    if args.verbose:
        print('---- Input Arguments ----', flush=True)
        for p in args._get_kwargs():
                print("{}: {}".format(p[0], p[1]), flush=True)
        print('---- --------------- ----', flush=True)

    if args.cmd == 'gribtonc':
        files_to_read = find_files_to_read(args)
        
        if args.filetype == 'grib2':
            datareader = readgrib(args, files_to_read)
            data = datareader.data
        else:
            print('Filetype: "{}", not supported.'.format(args.filetype), flush=True)
            sys.exit(1)

        nc_writer = writenc(args, data)


"""
Module containing simple standard statistics functions
"""
import numpy as np
from scipy.stats.stats import pearsonr

def bias(ref,obs):
    """calculates the bias between two arrays

    Parameters
    ----------
    ref : list
        Reference list
    obs : list
        Comparison list (e.g. Observations)

    Returns
    -------
    bias : float
        Bias from the two lists.
    """
    return np.mean(obs-ref)


def rmse(model,obs):
    """Calculates the Root Mean Square Error

    Parameters
    ----------
    model : list
        List containing model values
    obs : list
        List containing observations (reference)

    Returns
    -------
    rmse : float
        Root mean square error
    """

    N = np.count_nonzero(~np.isnan(obs))
    RMSE = np.sqrt(np.nansum((model-obs)**2)/(N-1))
    return RMSE


def variance(obs):
    """Calculates the variance for a list

    Parameters
    ----------
    obs : list
        list of values (eg. observations)

    Returns
    -------
    var : float
        variance of input list

    """
    return np.var(obs, ddof=1) # ddof = 1 > denominator=n-1


def std(obs):
    """Calculates the standard deviation

    Parameters
    ----------
    obs : list
        List with values (eg. observations)
    Returns
    -------
    std : float
        Standard deviation of input list
    """
    return np.sqrt(variance(obs))


def correlation(x,y):
    """Calculates the pearson correlation coefficient

    Parameters
    ----------
    x : list
        List of values
    y : list
        List of values

    Returns
    -------
    R : float
        Pearson correlation coefficient for input lists
    """
    return pearsonr(x,y)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tools for different geographical procedures often used.
"""
import numpy as np
from numpy import deg2rad, sin, cos, arctan2, sqrt, rad2deg, radians, arccos


__author__ = "Kasper Hintz"
__version__ = "0.1"
__email__ = "kah@dmi.dk"


def find_nearest_gridpoint(grid_lat, grid_lon, olat, olon, nx, ny, dx, dy, data):
    """Find nearest gridpoint in a regular grid

    Parameters
    ----------
    grid_lat : list
        List of latitudes in regular grid
    grid_lon : list
        List of longitudes in regular grid
    olat : float
        Latitude of point of interest
    olon : float
        Longitude of point of interest
    nx : integer
        Number of gridpoints in x-direction
    ny : integer
        Number of gridpoints in y-direction
    dx : float
        Horisontal resolution in degrees in x-direction
    dy : float
        Horisontal resolution in degrees in y-direction
    data : list
        List of data

    Returns
    -------
    datapoint : float
        The nearest value from data
    """
    i = int((olon - grid_lon[0]) / dx)
    j = int((olat - grid_lat[0]) / dy * nx)

    nearest_index = int((i % nx) + (j - (j % nx)))
    return data[nearest_index]


def point_inside_polygon(x, y, poly):
    """Checks if a point (x,y) is inside a polygon

    Parameters
    ----------
    x : float
        x-coordinate
    y : float
        y-coordinate
    poly : list
        List of tuples with points of the polygon

    Returns
    -------
    inside : boolean
        True if inside, False if outside
    """
    n = len(poly)
    inside = False

    p1x, p1y = poly[0]
    for i in range(n + 1):
        p2x, p2y = poly[i % n]
        if y > min(p1y, p2y):
            if y <= max(p1y, p2y):
                if x <= max(p1x, p2x):
                    if p1y != p2y:
                        xinters = (y - p1y) * (p2x - p1x) / (p2y - p1y) + p1x
                    if p1x == p2x or x <= xinters:
                        inside = not inside
        p1x, p1y = p2x, p2y

    return inside


def distance(lon1, lat1, lon2, lat2):
    """Calculate great-circle distance in kilometers

    Parameters
    ----------
    lon1 : float
        Longitude of first point in degrees
    lat1 : float
        Latitude of first point in degrees
    lon2 : float
        Longitude of second point in degrees
    lat2 : float
        Latitude of second point in degrees

    Returns
    -------
    d : float
        Great-circle distance in kilometers
    """
    pi = np.pi
    radius = 6371.  # Earth radius
    dlat = deg2rad(lat2 - lat1) / 2.0
    dlon = deg2rad(lon2 - lon1) / 2.0

    a = sin(dlat)**2 + cos(deg2rad(lat1)) * cos(deg2rad(lat2)) * sin(dlon)**2
    d = 2.0 * radius * arctan2(sqrt(a), sqrt(1 - a))
    return d

# def coordinate_from_distance(lon1, lat1, dx, dy):
#     """
#     Calculate the point from a reference dx, dy km away
#     dx<0 = westwards
#     dy<0 = southwards
#     """
#     pi = np.pi
#     radius = 6371.
#     lat = deg2rad(lat1)
#     lon = deg2rad(lon1)


def bearing(lon1, lat1, lon2, lat2):
    """Calculate bearing between two points

    Parameters
    ----------
    lon1 : float
        Longitude of first point in degrees
    lat1 : float
        Latitude of first point in degrees
    lon2 : float
        Longitude of second point in degrees
    lat2 : float
        Latitude of second point in degrees

    Returns
    -------
    bearing : float
        Bearing in radians
    """
    lat1 = deg2rad(lat1)
    lat2 = deg2rad(lat2)
    lon1 = deg2rad(lon1)
    lon2 = deg2rad(lon2)

    dlon = lon2 - lon1

    bearing = arctan2(sin(lon2 - lon1) * cos(lat2),
                      cos(lat1) * sin(lat2) - sin(lat1) * cos(lat2) * cos(lon2 - lon1))

    return bearing


def lon_lat_to_cartesian(lon, lat, R=1):
    """Calculates cartesian coordinates of a point on a sphere with radius R

    Parameters
    ----------
    lon : float
        Longitude
    lat : float
        Latitude
    R : float (optional)
        Radius of sphere, defaults to 1.

    Returns
    -------
    x : float
        x-coordinate
    y : float
        y-coordinate
    z : float
        z-coordinate
    """
    lon_r = radians(lon)
    lat_r = radians(lat)

    x = R * cos(lat_r) * cos(lon_r)
    y = R * cos(lat_r) * sin(lon_r)
    z = R * sin(lat_r)
    return x, y, z



"""
Module for IO relevant functions
"""
import sys
import os
import json
sys.path.insert(0, os.path.abspath('./dmitio/'))
from .arguments import arguments

def read_json(infile):
    """Reads data from a json file

    Parameters
    ----------
    infile : str
        Full path to json input file

    Returns
    -------
    data : dict
        dict with data
    """
    with open(infile, 'r') as f:
        try:
            data = json.load(f)
        except json.decoder.JSONDecodeError:
            data = []
    return data


def save_json(datadict,outfile):
    """Saves dict to a json file

    Parameters
    ----------
    datadict : dict
        json object (dict)
    outfile : str
        Full path to json output file
    """
    with open(outfile,'w') as f:
        json.dump(datadict,f)
    return



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Master module for dlotter.arguments
Called from dlotter.__main__
"""
import sys
import argparse
from argparse import ArgumentDefaultsHelpFormatter


class MyParser(argparse.ArgumentParser):
    def error(self, message):
        sys.stderr.write('error: %s\n' % message)
        self.print_help()
        sys.exit(2)


class arguments:

    def __init__(self) -> None:

        return
    
    
    def get_args(self, sysargs):

        parent_parser = MyParser(
            description='Quick and dirty conversion of NWP output',
            formatter_class=ArgumentDefaultsHelpFormatter)
        
        subparser = parent_parser.add_subparsers(dest="cmd")

        # Parser for NWP Deterministic
        parser_plot = subparser.add_parser('gribtonc', help='Convert GRIB to NETCDF')

        
        parser_plot.add_argument('-f',
                        '--filetype',
                        metavar='FILETYPE',
                        type=str,
                        help='What filetype are we using? (Options are: grib2)',
                        default='grib2',
                        required=False)
        
        parser_plot.add_argument('-d',
                        '--directory',
                        type=str,
                        help='directory to read data from',
                        default='.')
        
        parser_plot.add_argument('--prefix',
                        type=str,
                        help='Set to prefix of files if any',
                        default='',
                        required=False)

        parser_plot.add_argument('--postfix',
                        type=str,
                        help='Set to postfix of files if any',
                        default='',
                        required=False)
        
        parser_plot.add_argument('-o',
                        '--output-file',
                        metavar='OUTDIR',
                        type=str,
                        help='output file',
                        default='dmit.gribtonc.nc',
                        required=False)

        parser_plot.add_argument('-l',
                        '--limit-files',
                        metavar='LIMIT',
                        type=int,
                        help='Only use the first LIMIT files. If set to 0, not limit is used. If Limit > 0, files will be sorted by name first',
                        default=0,
                        required=False)

        parser_plot.add_argument('--verbose',
                        action='store_true',
                        help='Verbose output', 
                        default=False)


        if len(sysargs)==1:
            parent_parser.print_help()
            sys.exit(2)

        args = parent_parser.parse_args()

        return args


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Utility to send or fetch files to and from s3 bucket
"""
import boto
import boto.s3
import os
import traceback

try:
    host = 's3-%s.amazonaws.com' % os.environ['AWS_DEFAULT_REGION']
    S3 = boto.connect_s3(host=host)
    BUCKET = S3.get_bucket(os.environ['AWS_S3_BUCKET'])
except KeyError:
    pass


def burrito(description, job):
    """Utility for s3 transfer status
    """
    print(description, 'start', flush=True)
    try:
        job()
        print(description, 'done', flush=True)
        return True
    except:
        print(description, 'failed', flush=True)
        traceback.print_exc()
        return False


def put(path, key):
    """Takes a local file path and puts it into key in the bucket

    Parameters
    ----------
    path : str
        Local filename
    key : str
        Filename in s3 bucket

    Returns
    -------
    burrito : boolean
        Whether the transfer was successful (True) or failed (False)

    Examples
    --------
    >>> s3.put('local_example.png','s3_example.png')
    """
    return burrito('put %s' % path, lambda: boto.s3.key.Key(BUCKET, key).set_contents_from_filename(path))


def get(key, path):
    """Fetch a file from a s3 bucket puts it into local file

    Parameters
    ----------
    key : str
        Filename in s3 bucket
    path : str
        Local filename

    Returns
    -------
    burrito : boolean
        Whether the transfer was successful (True) or failed (False)

    Examples
    --------
    >>> s3.get('s3_example.png','local_example.png')
    """
    return burrito('get %s' % key, lambda: boto.s3.key.Key(BUCKET, key).get_contents_to_filename(path))


def list(prefix):
    """List files in s3 bucket

    Parameters
    ----------
    prefix : str
        s3 bucket path

    Returns
    -------
    bucket_list : list
        List of files in s3 bucket path
    """
    return BUCKET.list(prefix=prefix)



"""
Module for OS relevant functions
"""
import os
import time
import glob
import json

def find_files(directory, prefix="", postfix="", recursive=True, onlyfiles=True,
          fullpath=False, olderthan=None, inorder=False):
    """Find files in a directory.

    Parameters
    ----------
    directory : str
        Directory to search in
    prefix : str (optional)
        Only remove files with this prefix
    postfix : str (optional)
        Only remove files with the postfix
    recursive : Boolean (optional)
        Go into directories recursively. Defaults to True
    onlyfiles : Boolean (optional)
        Show only files. Defaults to True
    fullpath : Boolean (optional)
        Give full path. Defaults to False. If recursive=True, fullpath is given automatically.
    olderthan : int (optional)
        Match only files older than X seconds from now. Defaults to None
    inorder : Boolean (optional)
        Return sorted list of filenames. Defaults to False

    Returns
    -------
    files : list
        List containing file names that matches criterias

    Notes
    -----
    files = find_files('/foo/', prefix="", postfix="", recursive=False, onlyfiles=True, fullpath=True, olderthan=86400*100)
    """
    
    if recursive:
        fullpath = False
        files = []
        # r=root, d=directories, f = files
        for r, d, f in os.walk(directory):
            for file in f:
                if file.startswith(prefix) and file.endswith(postfix):
                    files.append(os.path.join(r, file))

    elif not recursive:

        if onlyfiles:
            files = [f for f in os.listdir(directory) if
                     f.endswith(postfix) and f.startswith(prefix) and
                     os.path.isfile(directory+f)]

        elif not onlyfiles:
            files = [f for f in os.listdir(directory) if
                     f.endswith(postfix) and f.startswith(prefix)]

    if fullpath:
        files = [directory+f for f in files]

    if olderthan is not None:
        now = time.time()
        tfiles = []
        for f in files:
            try:
                if not fullpath:
                    if os.path.getmtime(os.path.join(directory, f)) < (now - olderthan):
                        tfiles.append(f)
                else:
                    if os.path.getmtime(f) < (now - olderthan):
                        tfiles.append(f)
            except FileNotFoundError:
                continue
            
        files = tfiles

    if inorder:
        files = list(sorted(files))

    return files


def clean(files):
    """Removes files from the system.
    Note that filenames must be given as full paths

    Parameters
    ----------
    files : list
        List of files to remove.
    """
    for f in files:
        try:
            os.remove(f)
        except OSError:
            pass
    return


def does_file_exist(f):
    """
    Check if file exist

    Parameters
    ----------
    f : str
        name of file to check

    Returns
    -------
    state : boolean
        Whether the file exist (True) or not exist (False)
    """
    state = os.path.isfile(f)
    return state


def does_dir_exist(f):
    """
    Check if directory exist

    Parameters
    ----------
    f : str
        name of directory to check

    Returns
    -------
    state : boolean
        Whether the directory exist (True) or not exist (False)
    """
    state = os.path.exists(f)
    return state


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
 \nUtility to convert files from one format to another.
This is very much work in progress and thus not everything
should be expected to work perfect. 

Currently handled formats are:
- GRIB to NetCDF

It is important to set the ECCODES_DEFINITION_PATH correctly before running this.
Especially if you are using some local definitions. For example:

.. code-block:: sh

    export ECCODES_DEFINITION_PATH=/home/$USER/local_grib_definitions/definitions/:/home/$USER/miniconda3/envs/$ENV/share/eccodes/definitions/

This module is made to be run directly from command-line. For example:

.. code-block:: sh

    python -m dmit.dmitio.convert grib_to_nc --indir /home/$USER/gribfiles_dir/ -o test.nc
"""
import os
import sys
import traceback
import docopt
import netCDF4 as nc
import xarray as xr

from dmit import dmitio
from dmit import ostools
from dmit.dmitio import grib
from dmit.dmitio import netcdf

version = '0.0.1'

__doc__ += """
Usage:
  {filename} [grib_to_nc] [options]
  {filename} (-h | --help)
  {filename} --version

Commands:
  grib_to_nc                    Convert grib-file(s) to NetCDF.

Options:
  --namelist <str>              Specify namelist if not using default
  --indir <str>                 Specify input directory with files to convert. Directory should ONLY hold files to convert.
  -o, --output <str>            Specify output file [default: foo]      
  -h, --help                    Show this screen.
  -V, --version                 Show version
  --print-namelist              Print the namelist to get an example

""".format(filename=os.path.basename(__file__), version=version)


if __name__=="__main__":

    if len(sys.argv) < 2:
        print('Input missing: try with -h or --help')
        sys.exit(1)

    args = docopt.docopt(__doc__, version=str(version))

    print('Start converter')
    
    if not args['--namelist']:
        basedir="{}".format(os.path.abspath(__file__+'/..'))
        nml = "{}/{}".format(basedir,"convert.json")
    else:
        nml = args['--namelist']


    try:
        namelist = dmitio.read_json(nml)
    except FileNotFoundError:
        print('Namelist not found')
        sys.exit(1)


    if args['--print-namelist']:
        print(namelist)
        sys.exit(0) 

    if args['grib_to_nc']:

        outputfile = args['--output']

        gribfiles = ostools.find_files(args['--indir'],
                                       recursive=False,
                                       onlyfiles=True,
                                       fullpath=True,
                                       inorder=True)

        gribread = grib.grib_reader(namelist)
        data = gribread.get_data(gribfiles, namelist)
        print('- Got data from grib files\nProceed to writing')

        ncwrite = netcdf.netcdf_write(namelist)
        ncwrite.write_data(data, outputfile)
        print('Done writing to outputfile')


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
 \nUtility to read grib data
"""
import sys
import argparse
import eccodes as ec
import pygrib
import xarray as xr
import numpy as np
import datetime as dt

class readgrib:

    def __init__(self, args:argparse.Namespace, files_to_read:list) -> None:
        if args.verbose:
            print("Reading GRIB", flush=True)
        
        self.data = self.read(args, files_to_read)

        return


    def read(self, args:argparse.Namespace, files_to_read:list) -> None:

        grid_idx = 0 # Set to 0
        lats, lons = self.get_latlons(files_to_read[grid_idx])

        Nt = len(files_to_read)
        Nt_coords = np.zeros(Nt, dtype=dt.datetime)

        found_t_2_heightAboveGround_103  = False
        found_tp_0_heightAboveGround_103 = False

        for k,f in enumerate(files_to_read):
            gids = self.get_gids(f)

            time_gid = gids[0]

            ec.codes_set(time_gid, 'stepUnits', 'm')

            date = ec.codes_get(time_gid, 'dataDate')
            time = ec.codes_get(time_gid, 'dataTime')
            lead = ec.codes_get(time_gid, 'step')

            analysis = dt.datetime.strptime(('%i-%.2i')%(date,time),'%Y%m%d-%H%M')
            forecast = analysis + dt.timedelta(minutes=lead)
            Nt_coords[k] = forecast

            for i, gid in enumerate(gids):
                shortName = ec.codes_get(gid, 'shortName')
                level = ec.codes_get(gid, 'level')
                typeOfLevel = ec.codes_get(gid, 'typeOfLevel')
                levelType = ec.codes_get(gid, 'levelType')

                Ni = ec.codes_get(gid, 'Ni')
                Nj = ec.codes_get(gid, 'Nj')

                if args.verbose:
                    print(shortName,level,typeOfLevel,levelType, flush=True)

                if (shortName=='t' or shortName=='2t') and level==2 and \
                                        typeOfLevel=='heightAboveGround' and levelType=='103':
                    values = ec.codes_get_values(gid)
                    if not found_t_2_heightAboveGround_103: t2m = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)
                    found_t_2_heightAboveGround_103 = True
                    t2m[k,:,:] = values.reshape(Nj, Ni)
                
                if (shortName=='tp' or shortName=='tprate') and level==0 and \
                                        typeOfLevel=='heightAboveGround' and levelType=='103':
                    values = ec.codes_get_values(gid)
                    if not found_tp_0_heightAboveGround_103: precip = np.full([Nt,lats.shape[0],lons.shape[1]], np.nan)
                    found_tp_0_heightAboveGround_103 = True
                    precip[k,:,:] = values.reshape(Nj, Ni)

                ec.codes_release(gid)

        ds_grib = xr.Dataset(coords={"lat": (["x","y"], lats), 
                                     "lon": (["x","y"], lons), 
                                     "time": (["t"], Nt_coords)})

        if found_t_2_heightAboveGround_103: ds_grib['t2m'] = (['time', 'lat', 'lon'], t2m - 273.15 )
        if found_tp_0_heightAboveGround_103: ds_grib['precip'] = (['time', 'lat', 'lon'], precip )

        if len(list(ds_grib.data_vars)) == 0:
            raise SystemExit('No variables found. This can be due to missing tables in ECCODES_DEFINITION_PATH or that the requested keys are not yet implemented')
 
        ds_grib = self.sort_by_time(ds_grib)

        if found_tp_0_heightAboveGround_103:
            precip_accumulated = self.is_precip_accumulated(ds_grib['precip'])
            if precip_accumulated:
                ds_grib['precip'] = self.convert_accumulated_to_rate(ds_grib['precip'])

        return ds_grib


    def is_precip_accumulated(self, precip:xr.DataArray) -> bool:
        Nt = precip.shape[0]

        precip_accumulated = True

        psum = 0
        for k in range(Nt):
            csum = np.nansum(precip[k,:,:])
            if csum < psum: # If sum decreases precip can not be accumulated
                precip_accumulated = False
            psum = csum

        return precip_accumulated


    def convert_accumulated_to_rate(self, dataarray:xr.DataArray) -> xr.DataArray:

        Nt = dataarray.shape[0]
        converted_array = dataarray.copy(deep=True)

        for k in range(1,Nt):
            converted_array[k,:,:] = dataarray[k,:,:] - dataarray[k-1,:,:]

        return converted_array


    def sort_by_time(self, dataarray:xr.Dataset) -> xr.Dataset:

        nt = dataarray.dims['time']
        parameters = list(dataarray.data_vars)

        time = dataarray['time'].values
        idx = np.argsort(time)

        da = dataarray.sortby('time')

        for p in parameters:
            for k in range(nt):
                da[p][k,:,:] = dataarray[p][idx[k],:,:]
        
        return da


    def get_latlons(self, gribfile:str) -> tuple:
        """Get latitudes and longitudes from file. Uses pygrib as eccodes have no easy interface for that.

        Parameters
        ----------
        gribfile : str
            Path to gribfile

        Returns
        -------
        tuple
            tuple of latitudes, longitudes
        """

        gr = pygrib.open(gribfile)
        lats, lons = gr[1].latlons()
        gr.close()

        return lats, lons


    def get_gids(self, gribfile:str) -> list:
        """Get GribIDs (gid) for all the messages in one gribfile

        Parameters
        ----------
        gribfile : str
            path to gribfile

        Returns
        -------
        list
            list of grib-ids
        """
        
        f = open(gribfile, 'rb')
        msg_count = ec.codes_count_in_file(f)
        gids = [ec.codes_grib_new_from_file(f) for i in range(msg_count)]
        f.close()

        return gids

    # def find_keys(self, args:argparse.Namespace) -> None:
    #     return


# ---------- Kept only for backward compability ----------- #
class grib_reader:
    """Class to read and get data from gribfile
    DEPRECATED
    """

    def __init__(self, namelist):
        """Init for grib_reader

        Parameters
        ----------
        namelist : dict
            dictionary containing namelist
        """
        variables = [k for k in namelist['grib_translators'].keys()]

        self.search_t2m = True if "t2m" in variables else False
        self.search_tp  = True if "precip" in variables else False

        self.found_t2m = False
        self.found_tp = False
        
        return


    def get_latlons(self, gribfile):
        """Uses pygrib to get coordinates
        Parameters
        ----------
        gribfile : str
            path to gribfile

        Returns
        -------
        array
            array with latitudes
        array
            array with longitudes
        """
        gr = pygrib.open(gribfile)
        lats, lons = gr[1].latlons()
        gr.close()
        return lats, lons


    def get_data(self, gribfiles, namelist, stepunit="m"):
        """Get data from gribfiles according to a namelist

        Parameters
        ----------
        gribfiles : list
            list with paths to gribfiles
        namelist : dict
            namelist of job

        Returns
        -------
        xarray.Dataset
            Dataset with data from gribfile
        """

        gribfiles = list(gribfiles)

        lats, lons = self.get_latlons(gribfiles[0])

        Nt = len(gribfiles)
        time = np.empty(Nt, dtype='i8')

        init = True
        k=0
        for gribfile in gribfiles:

            f = open(gribfile, 'rb')

            while True:

                gid = ec.codes_grib_new_from_file(f)
                if gid is None:
                    break

                ec.codes_set(gid, 'stepUnits', stepunit)

                if init:
                    latdim = ec.codes_get(gid, 'Ny')
                    londim = ec.codes_get(gid, 'Nx')
                    if self.search_t2m: t2m = np.zeros([Nt,latdim,londim])
                    if self.search_tp: tp = np.zeros([Nt,latdim,londim])
                    init=False

                shortName = ec.codes_get(gid, 'shortName')
                level = ec.codes_get(gid, 'level')
                dataDate = ec.codes_get(gid, 'dataDate')
                dataTime = ec.codes_get(gid, 'dataTime')
                leadTime = ec.codes_get(gid, 'step')

                
                ct = dt.datetime.strptime('{}{:02d}'.format(dataDate,dataTime), '%Y%m%d%H%M') + dt.timedelta(minutes=leadTime)
                time[k] = int((ct - dt.datetime(1970,1,1,0)).total_seconds())

                if (self.search_t2m) and (shortName=='t') and (level==2):
                    values = ec.codes_get_values(gid)
                    t2m[k,:,:] = values.reshape((latdim,londim))
                    self.found_t2m = True
                if (self.search_tp) and (shortName=='tp') and (level==0):
                    values = ec.codes_get_values(gid)
                    tp[k,:,:] = values.reshape((latdim,londim))
                    self.found_tp = True
                
                    
                ec.codes_release(gid)

            f.close()
            k+=1


        ds = xr.Dataset(
            data_vars=dict(
            ),
                coords=dict(
                    lon=(["y", "x"], lons),
                    lat=(["y", "x"], lats),
                    time=time,
                    )
            )

        if self.found_t2m: 
            t2m[np.where(t2m==0)] = np.nan
            ds = ds.assign(t2m= (("time", "y", "x"), t2m))
        if self.found_tp: 
            ds = ds.assign(tp= (("time", "y", "x"), tp))

        ds.attrs['starttime'] = time[0]
        
        return ds





#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Module for sun-related calculations such as sunrise and sunset.
Adopted from:
https://stackoverflow.com/questions/19615350/calculate-sunrise-and-sunset-times-for-a-given-gps-coordinate-within-postgresql
"""
import sys
import math
import numpy as np
import datetime as dt
import warnings


def calc_sun_time(longitude, latitude, dl=dt.datetime.utcnow(), setrise='sunset', zenith=90.8):
    """Estimate sunset or sunrise in UTC time

    Parameters
    ----------
    longitude : float
        Longitude in degrees
    latitude : float
        Latitude in degrees
    dl : datetime.datetime (optional)
        Day of interest as datetime.datetime object. Defaults to utcnow().
    setrise : str (optional)
        'sunset' or 'sunrise'. Defaults to 'sunset'
    zenith : float (optional)
        Which zenith to use. Defaults to 90.8 degrees.

    Returns
    -------
    sun_dl : datetime.datetime
        datetime in which hour and minutes are either sunset or sunrise depending on input

    Examples
    --------
    >>> sunset = calcSunTime(12, 55, dt.datetime.utcnow(), setrise='sunset')
    >>> sunset = calcSunTime(12, 55, dt.datetime.utcnow(), setrise='sunrise')
    """
    if setrise == 'sunset':
        isRiseTime=False
    elif setrise == 'sunrise':
        isRiseTime=True
    else:
        raise SyntaxError('input argument "setrise" should be either "sunset" or "sunrise"')

    now = dl
    day, month, year = now.day, now.month, now.year

    TO_RAD = math.pi/180

    #1. Calculate the day of the year
    N1 = math.floor(275 * month / 9)
    N2 = math.floor((month + 9) / 12)
    N3 = (1 + math.floor((year - 4 * math.floor(year / 4) + 2) / 3))
    N = N1 - (N2 * N3) + day - 30

    #2. Convert the longitude to hour value and calculate an approximate time
    lngHour = longitude / 15

    if isRiseTime:
        t = N + ((6 - lngHour) / 24)
    else:
        t = N + ((18 - lngHour) / 24)

    #3. Calculate the Sun's mean anomaly
    M = (0.9856 * t) - 3.289

    #4. Calculate the Sun's true longitude
    L = M + (1.916 * math.sin(TO_RAD*M)) + (0.020 * math.sin(TO_RAD * 2 * M)) + 282.634
    L = L % 360

    #5a. calculate the Sun's right ascension

    RA = (1/TO_RAD) * math.atan(0.91764 * math.tan(TO_RAD*L))
    RA = RA % 360

    #5b. right ascension value needs to be in the same quadrant as L
    Lquadrant  = (math.floor( L/90)) * 90
    RAquadrant = (math.floor(RA/90)) * 90
    RA = RA + (Lquadrant - RAquadrant)

    #5c. right ascension value needs to be converted into hours
    RA = RA / 15

    #6. calculate the Sun's declination
    sinDec = 0.39782 * math.sin(TO_RAD*L)
    cosDec = math.cos(math.asin(sinDec))

    #7a. calculate the Sun's local hour angle
    cosH = (math.cos(TO_RAD*zenith) - (sinDec * math.sin(TO_RAD*latitude))) / (cosDec * math.cos(TO_RAD*latitude))

    if cosH > 1:
        warnings.warn('Sun never rises, returns None', RuntimeWarning)
        return None

    if cosH < -1:
        warnings.warn('Sun never sets, returns None', RuntimeWarning)
        return None

    #7b. finish calculating H and convert into hours
    if isRiseTime:
        H = 360 - (1/TO_RAD) * math.acos(cosH)
    else:
        H = (1/TO_RAD) * math.acos(cosH)

    H = H / 15

    #8. Calculate local mean time of rising/setting
    T = H + RA - (0.06571 * t) - 6.622

    #9. Adjust back to UTC
    UT = T - lngHour
    UT = UT % 24
    hr = int(UT) % 24
    min = int(round((UT - int(UT))*60,0))

    sun_dl = dt.datetime(year, month, day, hr, min)

    return sun_dl


if __name__ == '__main__':
    dl = calcSunTime(12, 55, dt.datetime.utcnow(), setrise='sunset', zenith=90.8)
    print(dl.strftime('%H:%M'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tools for computing ps and slp
"""
import sys
import numpy as np


def netatmo_pressure_correction(p0, h):
    """Convert SLP reported by Netatmo to Surface pressure.
    Necessary because Netatmo uses a very crude formula.

    Parameters
    ----------
    p0 : float
        Netatmo SLP in hPa
    h : float
        Altitude of station in meters

    Returns
    -------
    ps : float
        Surface pressure
    """
    L = 0.0065  # K/m
    cp = 1007.  # J/(kg*K)
    T0 = 288.15  # K
    g = 9.82  # m/s**2
    M = 0.0289644  # kg/mol
    R = 8.31447  # J/(mol*K)

    ps = p0 * (1 - L * h / T0)**(g * M / (R * L))

    return ps


def wmo_ps_to_slp(Ps, H, T, RH=None, Td=None):
    """Correction to Sea Level Pressure using WMO recommendations

    Parameters
    ----------
    T : float
        2 m Temperature at station in Celcius
    Ps : float
        Station pressure in hPa
    Hp : float
        Station elevation in gpm
    RH : float (optional)
        Relative Humidity
    Td : float (optional)
        2 m Dewpoint temperature at station in Celcius

    Returns
    -------
    slp : float
        Sea level pressure in hPa

    Notes
    -----
    Either RH or Td must be specified. Otherwise a very bad guess is made.

    Following:
    https://www.wmo.int/pages/prog/www/IMOP/meetings/SI/ET-Stand-1/Doc-10_Pressure-red.pdf
    """

    Hp = H  # meters:
    # For now it is assumed that geopotentiel meters are close to meters
    # (which) is nearly true for the boundary layer.

    if Td is None:
        if RH is None:
            print('WARNING, None of Td and RH are set\nContinuing with guessed numbers')
            Td = T - 3.  # 3 is chosen randomly


    if not RH is None:
        es = 6.11 * 10**((7.5 * T) / (237.3 + T))
        e = RH * es

        A = 17.625
        B = 243.04  # C
        C = 610.94  # Pa

        # http://climate.envsci.rutgers.edu/pdf/LawrenceRHdewpointBAMS.pdf
        Td = (B * np.log(e / C)) / (A - np.log(e / C))

    if not Td is None:
        # Temperatures in C
        e = 6.11 * 10**((7.5 * Td) / (237.3 + Td))
        es = 6.11 * 10**((7.5 * T) / (237.3 + T))
        RH = e / es

    a = 0.0065  # K/gpm
    Ch = 0.12  # K/hPa
    Kp = 0.0148275  # K/gpm
    g = 9.80665  # m/s**2
    R = 287.05  # J/kg/K

    Ts = 273.15 + T

    slp = Ps * np.exp((g * Hp / R) / (Ts + 0.5 * a * Hp + e * Ch))

    if H < 50:  # meters
        Tv = (273.15 + T) / (1 - 0.379 *
                             (6.11 * 10**((7.5 * Td) / (237.7 + Td)) / Ps))
        C = Ps * Hp / (29.27 * Tv)
        slp = Ps + C

    return slp


def altimeter_setting(ps, altitude):
    """Follows computation from Madaus and Mass 2017 p. 513.
    It computes the altimeter setting from bias corrected values

    Parameters
    ----------
    ps : float
        Surface pressure in hPa
    altitude : float
        Altitude in m

    Returns
    -------
    slp : float
        Sea level pressure in hPa
    """
    data = ps

    k1 = 8.4228807 * 10**(-5)
    k2 = 0.190284

    slp = np.zeros(len(data))
    counter = 0
    i = 0
    for P in data:
        h = altitude[i]
        if h == -999.:
            slp[i] = np.nan
        else:
            Palt = (P - 0.3) * (1 + k1 * h / ((P - 0.3)**k2))**(1 / k2)
            slp[i] = Palt
        i += 1

    return slp



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
 \nUtility to write netcdf files
"""
import sys
import argparse
import xarray as xr
import numpy as np
import numpy.ma as ma
import datetime as dt
import netCDF4 as nc
from scipy import spatial

class writenc:

    def __init__(self, args:argparse.Namespace, data:xr.Dataset) -> None:
        
        self.write_data(args, data)

        return


    def write_data(self, args:argparse.Namespace, data:xr.Dataset) -> None:

        nt = data.dims['time']
        nlat = data.dims['lat']
        nlon = data.dims['lon']
        variable_list = list(data.keys())

        analysis = data['time'][0].values
        analysis = dt.datetime.utcfromtimestamp(analysis.astype(int) * 1e-9)

        data['time'] = data['time'].astype(dt.datetime) / 10e8

        ncf = nc.Dataset(args.output_file,'w')
        ncf.institution = 'DMI'
        ncf.source = 'DMI'
        ncf.analysis = analysis.strftime('%Y-%m-%d %H:%M')
        ncf.Conventions = "CF-1.6"

        dim_time = ncf.createDimension("time", nt)
        dim_lat  = ncf.createDimension("latitude", nlat)
        dim_lon  = ncf.createDimension("longitude", nlon)

        ncvar_time = ncf.createVariable('time', np.float64, ('time'), zlib=True)
        ncvar_time[:] = data['time']
        ncvar_time.units = 'seconds since 1970-1-1 0:00:00'
        ncvar_time.coordinates = 'time'

        ncvar_lat = ncf.createVariable('latitude', np.float32, ('latitude','longitude'), zlib=True)
        ncvar_lat[:,:] = data['lat']
        ncvar_lat.long_name = 'latitude'
        ncvar_lat.units = 'degrees_north'
        ncvar_lat:standard_name = 'latitude'

        ncvar_lon = ncf.createVariable('longitude', np.float32, ('latitude','longitude'), zlib=True)
        ncvar_lon[:,:] = data['lon']
        ncvar_lon.long_name = 'longitude'
        ncvar_lon.units = 'degrees_east'
        ncvar_lat:standard_name = 'longitude'

        if 't2m' in variable_list:
            ncvar_t2m = ncf.createVariable('air_temperature_2m', np.float32, ('time', 'latitude', 'longitude'), zlib=True)
            ncvar_t2m[:,:,:] = data['t2m']
            ncvar_t2m.units = 'Celcius'
            ncvar_t2m.long_name = 'air_temperature_2m'
        if 'precip' in variable_list:
            ncvar_tp = ncf.createVariable('precipitation', np.float32, ('time', 'latitude', 'longitude'), zlib=True)
            ncvar_tp[:,:,:] = data['precip']
            ncvar_tp.units = 'mm'
            ncvar_tp.long_name = 'precipitation'

        
        ncf.close()

        return


# ---------- Kept only for backward compability ----------- #
class netcdf_write:
    """Class to write netCDF files
    """

    def __init__(self, namelist):
        """Init function for netcdf_write class

        Parameters
        ----------
        namelist : dict
            Dictionary holding namelist
        """
        self.write_t2m = False
        self.write_tp = False

        return


    def write_data(self, ds, outputfile):
        """Writes data to netCDF file

        Parameters
        ----------
        ds : xarray.Dataset
            Dataset holding data should be written
        outputfile : str
            Path of the outputfile
        """

        variable_list = list(ds.keys())

        if 't2m' in variable_list: self.write_t2m = True
        if 'tp' in variable_list: self.write_tp = True


        ncf = nc.Dataset(outputfile,'w')
        ncf.institution = 'DMI'
        ncf.source = 'DMI'
        ncf.start_time = ds.attrs['starttime']

        dim_time = ncf.createDimension("time", ds.dims['time'])
        dim_lat  = ncf.createDimension("latitude", ds.dims['y'])
        dim_lon  = ncf.createDimension("longitude", ds.dims['x'])

        ncvar_time = ncf.createVariable('time', np.float64, ('time'), zlib=True)
        ncvar_time[:] = ds['time']
        ncvar_time.units = 'seconds since 1970-1-1 0:00:00'
        ncvar_time.coordinates = 'time'

        ncvar_lat = ncf.createVariable('latitude', np.float32, ('latitude','longitude'), zlib=True)
        ncvar_lat[:,:] = ds['lat']
        ncvar_lat.long_name = 'latitude, south is negative'
        ncvar_lat.units = 'degrees'

        ncvar_lon = ncf.createVariable('longitude', np.float32, ('latitude','longitude'), zlib=True)
        ncvar_lon[:,:] = ds['lon']
        ncvar_lon.long_name = 'longitude, west is negative'
        ncvar_lon.units = 'degrees'


        if self.write_t2m:
            ncvar_t2m = ncf.createVariable('air_temperature_2m', np.float32, ('time', 'latitude', 'longitude'), zlib=True)
            ncvar_t2m[:,:,:] = ds['t2m']
            ncvar_t2m.units = 'K'
            ncvar_t2m.long_name = 'air_temperature_2m'
        if self.write_tp:
            ncvar_tp = ncf.createVariable('total_precipitation', np.float32, ('time', 'latitude', 'longitude'), zlib=True)
            ncvar_tp[:,:,:] = ds['tp']
            ncvar_tp.units = 'mm'
            ncvar_tp.long_name = 'total_precipitation'

        ncf.close()

        return


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Module for the Van Ulden and Holtslag 1983 paper:
A. A. M. Holtslag and A. P. Van Ulden, "A Simple Scheme for Daytime
Estimates of the Surface Fluxes from Routine Weather Data", Journal of climate
and applied meteorology, 1983, Vol. 22, No. 4.

This code was used in Hintz, et. al. (2020).

"""
import sys
import numpy as np
import datetime as dt

__author__ = "Kasper Hintz"
__credits__ = ["A. A. M. Holtslag", "A. P. Van Ulden"]
__version__ = "0.1"
__email__ = "kah@dmi.dk"



def rad_to_deg(rad):
    """Convert radians to degrees.

    Parameters
    ----------
    rad : float, list
        Radians

    Returns
    -------
    deg : float, numpy.array
        Degrees
    """
    deg = rad * 180. / np.pi
    return deg


def deg_to_rad(deg):
    """Convert degrees to radians.

    Parameters
    ----------
    deg : float, list
        Degrees

    Returns
    -------
    rad : float, numpy.array
        radians
    """
    rad = deg * np.pi / 180.
    return rad


def solar_elevation(M, D, H, lon, lat):
    """ Compute the solar elevation

    Parameters
    ----------
    M : integer
        Month of Year (1-12)
    D : integer
        Day of month (1-31)
    H : integer
        Hour of day in UTC
    lon : float
        Longitude
    lat : float
        Latitude

    Returns
    ----------
    phi : float
        Solar elevation in radians
    """

    lon = deg_to_rad(lon)
    lat = deg_to_rad(lat)

    d = 30 * (M - 1) + D

    SL = 4.871 + 0.0175 * d + 0.033 * np.sin(0.017 * d)
    delta = np.arcsin(0.398 * np.sin(SL))

    h = -lon + 0.043 * np.sin(2 * SL) - 0.033 * \
        np.sin(0.0175 * d) + 0.262 * H - np.pi

    sin_phi = np.sin(delta) * np.sin(lat) + \
        np.cos(delta) * np.cos(lat) * np.cos(h)
    phi = np.arcsin(sin_phi)
    #phi_deg = rad_to_deg(np.arcsin(sin_phi))
    return phi


def incoming_solar_radiation(phi, cloudcover):
    """
    Parameters
    ----------
    phi : float
        Solar elevation in radians
    cloudcover : float
        Total cloudcover in percentage (0-1)

    Returns
    ----------
    K : float
        Incoming solar radiation in W/m^2
    """

    # Using constants from De Bilt measurements
    a1 = 1041  # W/m^2
    a2 = -69  # W/m^2
    # Using constants from Hamburg measurements
    b1 = -0.75
    b2 = 3.4

    K0 = a1 * np.sin(phi) + a2

    if cloudcover == 0:
        K = K0

    if cloudcover > 0:
        N = cloudcover

        if N > 1:
            N = 1
            # print('WARNING: cloudcover was above 1 - setting to 1 (100 %)')

        K = K0 * (1 + b1 * N**b2)

    return K


def net_radiation(K, cloudcover, T, albedo=0.23):
    """Computes the net radiation based on incoming solar radiation, cloud cover
    and the temperature.

    Parameters
    ----------
    K : float
        Incoming solar radiation in W/m^2
    cloudcover : float
        In percent (0-1)
    T : float
        Temperature in Kelvin
    albedo : float (optional)
        Albedo of the surface

    Other Parameters
    ----------------
    L_plus : float (internal)
        Incoming longwave
    L_minus : float (internal)
        Outgoing longwave

    Returns
    -------
    Q : float
        Net radiation in W/m^2
    """

    if cloudcover > 1:
        cloudcover = 1
    if cloudcover < 0:
        cloudcover = 0

    c1 = 5.31 * 10**(-13)  # W m**-2 K**-6
    c2 = 60  # W m**-2
    c3 = 0.12
    sigma = 5.67 * 10**(-8)  # W m**-2 K**-4 Stefan Boltzmann constant

    L_plus = c1 * T**6 + c2 * cloudcover
    #L_minus = sigma * T**4 + c3*Q

    Q = ((1 - albedo) * K + L_plus - sigma * T**4) / (1 + c3)

    return Q


def surface_energy_budget(Q, T):
    """
    Computes the Sensible, Latent and Soil heat flux

    Parameters
    ----------
    Q  : float
        Net Radiation
    T  : float
        Temperature in Kelvin

    Returns
    -------
    H : float
        Sensible Heat Flux
    E : float
        Latent Heat Flux
    G : float
        Soil Heat Flux
    """
    cg = 0.1
    G = cg * Q

    Q_remain = Q - G

    alpha = 1  # W m**-2 (Grass Surface)
    B = 20  # W m**-2 (Grass Surface)
    gamma_s = gamma_s_table(T - 273.15)

    # Eq. 14
    H = ((1 - alpha) + gamma_s) / (1 + gamma_s) * Q_remain - B
    # Eq. 15
    E = alpha / (1 + gamma_s) * Q_remain + B

    return H, E, G


def gamma_s_table(T):
    """Table 5 from Holtslag and Van Ulden (1983)

    Parameters
    ----------
    T : float
        Temperature in Celcius

    Returns
    -------
    gamma_s: float
        Temperature dependent variable
    """
    base = 5
    Tb = base * round(T / base)
    if Tb < -5:
        Tb = -5
    if Tb > 35:
        Tb = 35

    if Tb == -5:
        gamma_s = 2.01
    if Tb == 0:
        gamma_s = 1.44
    if Tb == 5:
        gamma_s = 1.06
    if Tb == 10:
        gamma_s = 0.79
    if Tb == 15:
        gamma_s = 0.60
    if Tb == 20:
        gamma_s = 0.45
    if Tb == 25:
        gamma_s = 0.35
    if Tb == 30:
        gamma_s = 0.27
    if Tb == 35:
        gamma_s = 0.21

    return gamma_s


def momemtum_flux_and_obukhov_length(U, z, z0, T, H):
    """
    Parameters
    ----------
    U : float
        Wind speed at height z
    z : float
        Height above surface of measured wind speed
    z0 : float
        Roughness length
    T : float
        Temperature in Kelvin
    H : float
        Sensible heat flux in W/m^2

    Returns
    -------
    u_star : float
        Friction velocity (Momentum flux)
    L : float
        Obukhov Length (Stability parameter)

    Notes
    -----
    u_star and L is solved in an iterative process
    """

    eps = sys.float_info.epsilon

    k = 0.41
    g = 9.81
    rho = 1.225  # kg m**-3 (at 15 degrees)
    cp = 1004  # J * kg**-1 * K**-1

    L_init = -36

    do_proceed = True
    iter = 0
    L = L_init

    while do_proceed:
        iter += 1

        L_old = L

        if L<0:
            x = (1 - 16 * z / L)**(1 / 4)
            x0 = (1 - 16 * z0 / L)**(1 / 4)

            phi_m_z = 2 * np.log((1 + x) / 2) + \
                np.log((1 + x**2) / 2) - 2 * np.arctan(x) + np.pi / 2

            phi_m_z0 = 2 * np.log((1 + x0) / 2) + \
                np.log((1 + x0**2) / 2) - 2 * np.arctan(x0) + np.pi / 2

        elif L>=0:
            phi_m_z = - 5 * z / L
            phi_m_z0 = - 5 * z0 / L

        if iter == 1:
            phi_m_z = phi_m_z0 = 0

        u_star = k * U * (np.log(z / z0) - phi_m_z + phi_m_z0)**(-1)

        L = - (rho * cp * T * u_star**3) / (k * g * H)

        if abs(L) < eps:
            do_proceed = False
        elif L == 0:
            do_proceed = False
        elif abs(1 - L_old / L) < 0.05:
            do_proceed = False

        if not L == L:
            do_proceed = False

    return u_star, L


def vanulden_stability(dl, lat, lon, cloudcover, temperature, wspd, z, z0):
    """ Main function to collect and return and the Van Ulden
    and Holtslag functions to compute momentum flux and Obukhov length.

    Parameters
    ----------
    dl : datetime.datetime object
        Time in utc (e.g. datetime.datetime.utcnow())
    lat : float
        Latitude in degress
    lon : float
        Longitude in degrees
    cloudcover: float
        Cloudcover in percentage (0-1)
    temperature : float
        Temperature in Kelvin
    wspd : float
        Wind speed reference Observation in m/s
    z : float
        Elevation of wind measurement in m
    z0 : float
        Roughness length

    Returns
    -------
    u_star : float
        Friction velocity
    L : float
        Obukhov length
    """

    M = int(dl.strftime('%-m'))
    D = int(dl.strftime('%-d'))
    H = int(dl.strftime('%-H'))

    phi = solar_elevation(M, D, H, lon, lat)
    K = incoming_solar_radiation(phi, cloudcover)
    Q = net_radiation(K, cloudcover, temperature)

    sens_heat, lat_heat, soil_heat = surface_energy_budget(Q, temperature)

    u_star, L = momemtum_flux_and_obukhov_length(
        wspd, z, z0, temperature, sens_heat)

    return u_star, L


if __name__ == '__main__':

    # dl = dt.datetime(2016, 2, 14, 12)
    #
    # lon = 10
    # lat = 55
    #
    # cloudcover = 2 / 8  # Octas
    # temperature = 283.15  # K
    # U = 8  # m/s
    # z = 10  # m
    # z0 = 0.1  # m

    #
    dl =  dt.datetime(2020,5,25,0,20,0)
    lat = 55.73567
    lon = 11.603826
    cloudcover = 0.0011596679687500002  #%
    temperature = 274.026145   # Kelvin
    U = 10  # m/s
    z = 10  #m
    z0 = 0.08139686637750906

    u_star, L = vanulden_stability(
        dl, lat, lon, cloudcover, temperature, U, z, z0)



import sys
import numpy as np
from scipy import stats as ss

def calculate_fss(obs,mod,percentile=None,threshold=None,nmax=50):
    """Calculates FSS

    Parameters
    ----------
    obs : array
        Input array of truth (eg. observations)
    mod : array
        Input array of model (eg. model data)
    percentile : float (optional)
        Which percentile to use. Defaults to None
    threshold : float (optional)
        Which threshold to use. Defaults to None
    nmax : integer (optional)
        How many gridpoints to consider. Defaults to 50.

    Returns
    -------
    FSS : list
        List with FSS for different scales
    f0 : float
        FSS value for random values

    Notes
    -----
    Either percentile or threshold should be set.
    """

    if (percentile == None) and (threshold==None):
        sys.exit('All input to calculate_fss was None')

    FSS_NMAX  = nmax
    FSS_NSTOP = int(FSS_NMAX/2+1)
    FSS_RANGE = range(1,FSS_NMAX+2,2)
    FSS = np.zeros(FSS_NSTOP)

    # FZERO = (100. - percentile)/100. # Random Forecast FSS

    OBS_BIN = np.zeros(obs.shape)
    MOD_BIN = np.zeros(mod.shape)

    NY, NX = obs.shape

    # Create Binary Field
    if threshold==None:
        print('Using percentile', flush=True)
        OBS_CUTOFF = ss.scoreatpercentile(obs.flatten(),percentile)
        MOD_CUTOFF = ss.scoreatpercentile(mod.flatten(),percentile)
    if percentile==None:
        print('Using threshold', flush=True)
        OBS_CUTOFF = threshold
        MOD_CUTOFF = threshold

    OBS_WET = obs >= OBS_CUTOFF
    MOD_WET = mod >= MOD_CUTOFF

    OBS_BIN[OBS_WET] = 1.
    MOD_BIN[MOD_WET] = 1.

    f0 =  np.sum(OBS_BIN) / len(OBS_BIN.flatten())

    print('FSS_RANDOM = '+str(f0), flush=True)

    print('OBS,CUTOFF:'+str(OBS_CUTOFF),'MOD,CUTOFF:'+str(MOD_CUTOFF), flush=True)
    print('OBS,SUM:'+str(np.sum(OBS_BIN)),'MOD,SUM:'+str(np.sum(MOD_BIN)), flush=True)

    OBS_FRACTION = np.zeros(obs.shape)
    MOD_FRACTION = np.zeros(mod.shape)

    # print('Percent done with outer loop, cpu time for outer loop iteration')
    for i in range(FSS_NSTOP):

        n = FSS_RANGE[i]

        for iy in range(0,NY):
            YRS = int(iy-(n-1)/2)   # start
            YRE = int(iy-(n-1)/2+n) # end
            if YRS<0: YRS = 0
            if YRE>NY: YRE = NY

            for ix in range(0,NX):
                XRS = int(ix-(n-1)/2)
                XRE = int(ix-(n-1)+n)
                if XRS < 0: XRS = 0
                if XRE > NX: XRE = NX

                OBS_FRACTION[iy,ix] = 1./(n*n) * np.sum(OBS_BIN[YRS:YRE,XRS:XRE])
                MOD_FRACTION[iy,ix] = 1./(n*n) * np.sum(MOD_BIN[YRS:YRE,XRS:XRE])

        OBS_PROVINCIAL = np.sum(OBS_FRACTION*OBS_FRACTION)
        MOD_PROVINCIAL = np.sum(MOD_FRACTION*MOD_FRACTION)

        MSE     = np.sum((OBS_FRACTION-MOD_FRACTION)**2)
        MSE     = MSE*1./(NX*NY)

        MSE_REF = 1./(NX*NY) * (MOD_PROVINCIAL+OBS_PROVINCIAL)

        if MSE_REF != 0.:
            FSS[i] = 1. - (float(MSE)/float(MSE_REF))


    return FSS, f0



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
     name='dmit',
     version='0.4.5',
     author="Kasper Hintz",
     author_email="kah@dmi.dk",
     description="A Met Tool package",
     long_description=long_description,
     long_description_content_type="text/markdown",
     packages=setuptools.find_packages(),
     setup_requires=['wheel'],
     classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ],
     install_requires=[
         'wheel',
         'xarray',
         'numpy',
         'eccodes',
         'pygrib',
         ],
     include_package_data=True,
     data_files=[('dmit', ['dmit/dmitio/convert.json'])],
 )



"""
Module for manipulating dates
"""

import datetime
from calendar import monthrange
from datetime import date, datetime, timedelta
from typing import Union

def round_time(dt:datetime = None, roundTo:timedelta = timedelta(minutes=1), roundType:str = 'nearest') -> datetime:
    """Round a time to a nearest common format

    Parameters
    ----------
    dt : datetime.datetime object
        Datetime one wish to round
    roundTo : datetime.timedelta object (optional)
        What to round to. Defaults to datetime.timedelta(minutes=1).
    roundType : str
        How to round. 'nearest', 'roundup', 'rounddown'

    Returns
    -------
    dt_rounded : datetime.datetime object
        Rounded datetime
    """
    roundTo = roundTo.total_seconds()
    if dt == None : dt = datetime.datetime.now()
    seconds = (dt - dt.min).seconds
    if roundType=='nearest':
        rounding = (seconds+roundTo/2) // roundTo * roundTo
    elif roundType=='roundup':
        rounding = (seconds+roundTo) // roundTo * roundTo
    elif roundType=='rounddown':
        rounding = seconds // roundTo * roundTo

    return dt + timedelta(0, rounding - seconds, -dt.microsecond)


def per_delta(start:datetime, end:datetime, delta:timedelta):
    """Generate a list of datetimes within an interval

    Parameters
    ----------
    start : datetime.datetime object
        Starting date
    end : datetime.datetime object
        End date
    delta : datetime.timedelta object
        Increment in time, e.g. 1 hour would be datetime.timedelta(hours=1)
    """
    curr = start
    while curr < end:
        yield curr
        curr += delta


# def current_run_datetime(frequency:int=3, delay:int=2) -> datetime:
#     """Get the time of the current valid analysis

#     Parameters
#     ----------
#     frequency : integer (optional)
#         Frequency of analysis in hours. Defaults to 3 hours.
#     delay : integer (optional)
#         Delay in hours between now and when analysis becomes available. Defaults to 2 hours.

#     Returns
#     -------
#     nowrounded : datetime.datetime object
#         Time for the latest current valid analysis
#     """
#     now = datetime.utcnow()
#     nowrounded = now.replace(hour=((now.hour - delay) - (now.hour - delay) % frequency) % 24)
#     if (now - timedelta(hours=frequency)).strftime('%d') != nowrounded.strftime('%d'):
#         nowrounded = nowrounded - timedelta(hours=24)
#     return nowrounded

def current_run_datetime(frequency:int=180, delay:int=120, roundto:str='hour', now:Union[datetime, None]=None) -> datetime:
    """Get the time of the current valid analysis

    Parameters
    ----------
    frequency : integer (optional)
        Frequency of analysis in minutes. Defaults to 180 minutes (3 hours).
    delay : integer (optional)
        Delay in minutes between now and when analysis becomes available. Defaults to 120 minutes (2 hours)
    roundto : string (optional)
        What to round to: Available is "hour", "minute". Defaults to "hour"
    now : datetime object (optional)
        Current time to get run for. Defaults to now.

    Returns
    -------
    nowrounded : datetime.datetime object
        Time for the latest current valid analysis
    """

    if now is None: now=datetime.utcnow()

    if roundto=='hour':
        delay = delay//60
        frequency = frequency//60

        delayed_now_hour = now.hour - delay

        if delayed_now_hour<0: now += timedelta(days=delayed_now_hour//24)
        nowrounded = now.replace(hour=((delayed_now_hour) - (delayed_now_hour) % frequency) % 24)

    elif roundto=='minute' or roundto=='10minute':
        if int(now.minute) < delay:
            delay+=60 
        nowrounded = now.replace(minute=((now.minute - delay) - (now.minute - delay) % frequency) % 60)
        nowrounded = nowrounded.replace(hour=(nowrounded.hour - (nowrounded.minute)//50) % 24)

    if (now - timedelta(minutes=frequency)).strftime('%d') != nowrounded.strftime('%d'):
        nowrounded = nowrounded - timedelta(hours=24)

    if roundto=='hour':
        nowrounded = round_time(nowrounded, roundTo=timedelta(hours=1), roundType='rounddown')
    elif roundto=='minute':
        nowrounded = round_time(nowrounded, roundTo=timedelta(minutes=1), roundType='rounddown')
    elif roundto=='10minute':
        nowrounded = round_time(nowrounded, roundTo=timedelta(minutes=10), roundType='rounddown')

    return nowrounded


def month_delta(d1:datetime, d2:datetime) -> int:
    """Find the difference between dates in months

    Parameters
    ----------
    d1 : datetime.datetime object
        Date 1
    d2 : datetime.datime object
        Date 2

    Returns
    -------
    delta : integer
        Difference between dates in months
    """
    delta = 0
    while True:
        mdays = monthrange(d1.year, d1.month)[1]
        d1 += timedelta(days=mdays)
        if d1 <= d2:
            delta += 1
        else:
            break
    return delta


def add_month(date=None):
    """Add one month to date

    Parameters
    ----------
    date : datetime.datetime object
        Date to add one month to.

    Returns
    -------
    candidate: datetime.datetime object
        Candidate date with one month added
    """
    # number of days this month
    month_days = monthrange(date.year, date.month)[1]
    candidate = date + timedelta(days=month_days)
    # but maybe we are a month too far
    if candidate.day != date.day:
        # go to last day of next month,
        # by getting one day before begin of candidate month
        return candidate.replace(day=1) - timedelta(days=1)
    else:
        return candidate


def subtract_one_month(date=None):
    """Subtract one month from date

    Parameters
    ----------
    date : datetime.datetime object
        Date to subtract one month from.

    Returns
    -------
    dt3: datetime.datetime object
        Date after subtracting one month
    """
    dt1 = date.replace(day=1)
    dt2 = dt1 - timedelta(days=1)
    dt3 = dt2.replace(day=1)
    return dt3


def count_days(date1=None, date2=None, includeEnd=False):
    """Count the days between two dates

    Parameters
    ----------
    date1 : datetime.datetime object
        Date 1
    date2 : datetime.datetime object
        Date 2
    includeEnd : boolean
        Whether to include the last day itself. Defaults to False.

    Returns
    -------
    days : integer
        Days between two dates.
    """
    if includeEnd:
        return int((date2-date1).total_seconds()/(60*60*24) + 1)
    else:
        return int((date2-date1).total_seconds()/(60*60*24))
